local BLD_MODE_WORK_RES = fv.core.BLD_MODE_WORK_RES
local BLD_STATUS_PASTURE_SLAY = fv.core.BLD_STATUS_PASTURE_SLAY
local RES_WATER = fv.core.RES_WATER
local BUILDING_NO_BARN = fv.core.BUILDING_NO_BARN
local HOUSE_FREE_CELL = fv.core.HOUSE_FREE_CELL
local BUILDING_NO_SATIETY = fv.core.BUILDING_NO_SATIETY
local BUILDING_NO_WATER = fv.core.BUILDING_NO_WATER
local BUILDING_NO_BARN = fv.core.BUILDING_NO_BARN
local config = fv.core.config
local math = require("math")

local function findAndTransferFood(w, b, animalFood, satietyNow, needSatiety, animalConfig)
	local foodWarning = false
	local maxGetCount = getMaxTransportCount(w, animalFood)
	-- local transformCount = math.min(math.floor((needSatiety-satietyNow)/animalConfig.satietyPerFood), maxGetCount)
	-- if transformCount >= maxGetCount or satietyNow/needSatiety < 0.3 then


	if satietyNow/needSatiety < 0.5 then
		if bringFromNearestBuilding(w, b, animalFood, maxGetCount, true, config.findResourcesRadius) then
			b:hideIcon(BUILDING_NO_SATIETY)
			return true;
		else
			if satietyNow == 0 then
				foodWarning = true
			end
		end		
	end
	
	b:setVisibleIcon(BUILDING_NO_SATIETY, foodWarning)
	return false
end
		
local function findAndTransferWater(w, b, thirstNow, needThirst, animalConfig)
	local waterWarning = false
	local maxGetCount = getMaxTransportCount(w, RES_WATER)
	-- local transformCount = getMaxTransportCount(w, RES_WATER)
	if thirstNow/needThirst < 0.5 then
		if bringFromNearestBuilding(w, b, RES_WATER, maxGetCount, false, config.findResourcesRadius) then						
			b:hideIcon(BUILDING_NO_WATER)			
			return true;
		else 
			if thirstNow == 0 then
				waterWarning = true
			end
		end
	end				
	
	b:setVisibleIcon(BUILDING_NO_WATER, waterWarning)
	return false
end
	
local function slayAnimal(w, b)
	local slaughterAnimal = nil
	local ageMax = -1
	local animalCount = b.animalsCount
	for i = 0, animalCount-1, 1 do
		local animal = b:getAnimal(i)
		if animal ~= nil then		
			if animal.age > ageMax and animal.reserved == false then --if animal.age > ageMax and isAnimalReserved(animal) == false then	
				ageMax = animal.age
				slaughterAnimal = animal
			end
		end
	end
		
	if slaughterAnimal ~= nil then
		w:addTaskSetHint("@hint_slay_animal@")
		
		w:addTaskReserveAnimal(slaughterAnimal, true)
		--w:addTaskPlayAnimation("forward")		
		w:addTaskWalkToLiveObject(slaughterAnimal, "forward", false)
		w:addTaskPlayAnimation("stay")
		local data = {}
		data["uid"] = slaughterAnimal.uid
		data["bld_uid"] = b.uid
		data["state"] = 0
		data["lootOut"] = 0
		w:addTaskScript("KillAnimalTask", data)
		w:addTaskReserveAnimal(slaughterAnimal, false)
		b:resetWorkStatus(BLD_STATUS_PASTURE_SLAY)
		
		w:addTaskResetHint()
		return true
	end
	b:resetWorkStatus(BLD_STATUS_PASTURE_SLAY)
	return false
end

function fv.core.Pasture.updateWorker(b, w)  

	if repair(w, b) then
		return true
	end
	
	local pastureBuild = b
	local animalCount = pastureBuild.animalsCount
	
	local animalFood = ""
	
	local animalConfig = pastureBuild.animalCfg
	if animalConfig ~= nil then
		animalFood = animalConfig.foodResource
	end
	
	local limited = resourceLimitReached(b:getWorkMode(BLD_MODE_WORK_RES))
	
	local wildanimal = findMostDangerAnimal(w, b:getDoorPosition(w), 14)
	if wildanimal ~= nil then
		w:addTaskSetHint("@actHelp_kill_animal@")
			
		w:addTaskReserveAnimal(wildanimal, true)
		w:addTaskWalkToLiveObject(wildanimal, "forward", false)		
		local scriptData = {}
		scriptData["uid"] = wildanimal.uid
		scriptData["bld_uid"] = b.uid
		scriptData["state"] = 0
		scriptData["lootOut"] = 1
		w:addTaskScript("KillAnimalTask", scriptData)
		w:addTaskReserveAnimal(wildanimal, false)
		
		w:addTaskResetHint()
		return true
	end

	-- вначале забиваем
	local slayState = false
	if b:getWorkStatus(BLD_STATUS_PASTURE_SLAY) then
		slayState = true		
	end	
	
	local slayOldAnimal = false
	local breedAnimalsCount = 0
	local breedAnimalsNum = {}
	for i = 0, animalCount-1, 1 do
		local animal = pastureBuild:getAnimal(i)
		if animal ~= nil then
			
			if limited == false then
				if animalCount > 1 then
					if animal.age >= animalConfig.ageSlaughtering then
						if animal.reserved == false then --if isAnimalReserved(animal) == false then
							slayState = true
						else
							slayOldAnimal = true
						end
					end
				end
			end
		
			if animal.age >= animalConfig.ageGrown then	
				breedAnimalsNum[breedAnimalsCount] = i
				breedAnimalsCount = breedAnimalsCount + 1
			end
		end
	end
	
	if slayState == false then
		if animalCount >= pastureBuild.maxAnimalsCount and slayOldAnimal == false then
			slayState = true
		end
	end
	
	if slayState then
		if slayAnimal(w, b) then
			return
		end
	end

	-- относим добытое
	local res = getBuildingResources(b)	
	for key,value in pairs(res) do
		if (key ~= animalFood or animalCount == 0) and key ~= RES_WATER then
			value = math.max(value + b:getPickReservation(key), 0)
			local maxGetCount = getMaxTransportCount(w, key)
			if value >= maxGetCount or animalCount == 0 then
				local transportCount = math.min(value, maxGetCount)	
				if transportCount > 0 then
					if storeToNearestBuilding(w, b, key, transportCount) then
						b:hideIcon(BUILDING_NO_BARN)
						return true;
					else
						b:showIcon(BUILDING_NO_BARN)
					end	
				end				
			end
		end
	end
	
	if animalCount > 0 and animalConfig then
		-- следим за водой
		local futureWaterCount = b:getResourceFree(RES_WATER) + b:getPutReservation(RES_WATER)
		local thirstNow = futureWaterCount*animalConfig.thirstPerWater
		local needThirst = animalCount*animalConfig.thirstMax
	
		-- следим за едой
		local futureCount = b:getResourceFree(animalFood) + b:getPutReservation(animalFood)
		local satietyNow = futureCount*animalConfig.satietyPerFood
		local needSatiety = animalCount*animalConfig.satietyMax
		
		local foodTransfer = false		
		if satietyNow < needSatiety and needSatiety > 0 then
			foodTransfer = true
		end
		local waterTransfer = false
		if thirstNow < needThirst and needThirst > 0 then
			waterTransfer = true
		end		
		--[[
		if foodTransfer and waterTransfer then
			if satietyNow/needSatiety < thirstNow/needThirst then
				if findAndTransferWater(w, b, thirstNow, needThirst, animalConfig) then
					return true
				end
				if findAndTransferFood(w, b, animalFood, satietyNow, needSatiety, animalConfig) then
					return true
				end
			else
				if findAndTransferFood(w, b, animalFood, satietyNow, needSatiety, animalConfig) then
					return true
				end
				if findAndTransferWater(w, b, thirstNow, needThirst, animalConfig) then
					return true
				end
			end
		else
		]]
			if foodTransfer then
				if findAndTransferFood(w, b, animalFood, satietyNow, needSatiety, animalConfig) then
					return true
				end
			end		
			if waterTransfer then
				if findAndTransferWater(w, b, thirstNow, needThirst, animalConfig) then
					return true
				end
			end
		-- end
	end

	-- помогаем размножаться
	if breedAnimalsCount > 0 then
		local randomAnimalNum = breedAnimalsNum[math.random(breedAnimalsCount)-1]
		local breedingAnimal = pastureBuild:getAnimal(randomAnimalNum)
		if breedingAnimal then
			w:addTaskSetHint("@hint_breed_animal@")
			
			--w:addTaskPlayAnimation("forward")
			w:addTaskWalkToLiveObject(breedingAnimal, "forward", false)
			w:addTaskWork(3000, "stay")
			
			local scriptData = {}
			scriptData["uid"] = breedingAnimal.uid
			scriptData["breed"] = 3000
			w:addTaskScript("taskBreedAnimal", scriptData)
			
			w:addTaskResetHint()
			return;
		end
	end

	-- гуляем
	if animalCount > 0 then
		local wayPoint = findRandPoint(b.pos, math.min(b.lengthDefault, b.widthDefault)*0.5, HOUSE_FREE_CELL)
		if canWalkToPoint(w, wayPoint) then
			goTo(w, wayPoint)
			w:addTaskPlayAnimation("stay")
			w:addTaskWait(3000)
			return;
		end	
	end
end

fv.core.Chicken_pasture.updateWorker = fv.core.Pasture.updateWorker
fv.core.Cow_pasture.updateWorker = fv.core.Pasture.updateWorker
fv.core.Pig_pasture.updateWorker = fv.core.Pasture.updateWorker
fv.core.Sheep_pasture.updateWorker = fv.core.Pasture.updateWorker

